"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.guard = void 0;
const symbols_1 = require("./symbols");
const utils_1 = require("./utils");
function guard(name, fn, handleOptions, defaultOptions = {}) {
    var _a, _b;
    const transform = ((_a = handleOptions === null || handleOptions === void 0 ? void 0 : handleOptions.transform) !== null && _a !== void 0 ? _a : utils_1.identity);
    const validate = ((_b = handleOptions === null || handleOptions === void 0 ? void 0 : handleOptions.validate) !== null && _b !== void 0 ? _b : utils_1.trueFn);
    function getFnWithErrors(options) {
        return (value, path = []) => {
            const data = fn(value, options);
            if (!(0, utils_1.isValid)(data)) {
                return {
                    valid: false,
                    data: undefined,
                    errors: [
                        {
                            message: `vality.${name}.base`,
                            path,
                            options,
                            value,
                        },
                    ],
                };
            }
            if (!validate(data)) {
                return {
                    valid: false,
                    data: undefined,
                    errors: [
                        {
                            message: `vality.${name}.custom`,
                            path,
                            options,
                            value,
                        },
                    ],
                };
            }
            if (handleOptions === undefined)
                return { valid: true, data: transform(data), errors: [] };
            const keysWithError = Object.keys(options).filter(k => k !== "transform" &&
                k !== "validate" &&
                handleOptions[k] !== undefined &&
                !handleOptions[k](data, options[k], options));
            if (keysWithError.length === 0) {
                return {
                    valid: true,
                    data: transform(data),
                    errors: [],
                };
            }
            return {
                valid: false,
                data: undefined,
                errors: keysWithError.map(k => ({
                    message: `vality.${name}.options.${k}`,
                    options,
                    path,
                    value,
                })),
            };
        };
    }
    return Object.assign((options) => {
        return {
            [symbols_1._validate]: getFnWithErrors(Object.assign(Object.assign({}, defaultOptions), options)),
            [symbols_1._options]: undefined,
            [symbols_1._type]: undefined,
        };
    }, {
        [symbols_1._validate]: getFnWithErrors(defaultOptions),
        [symbols_1._options]: undefined,
        [symbols_1._type]: undefined,
    });
}
exports.guard = guard;
